'coded by UEZ build 2016-08-04
'idea taken from http://rosettacode.org/wiki/Mandelbrot_set#C.23 / JS version

#include "windows.bi"
#include "fbgfx.bi"
Using FB

Declare Function mandelIter(cx As Single, cy As Single, maxIter As ULong) As ULong
Declare Sub Mandelbrot(xmin As Single, xmax As Single, ymin As Single, ymax As Single, iterations As ULong, iDisplayFrequency As ULong = 100)

Dim As Integer sW, sH
ScreenInfo(sW, sH)

Dim Shared As ULong iW, iH

iW = 1000 'Int(sW * 0.95)
iH = 600 'Int(sH * 0.90)

ScreenControl FB.SET_DRIVER_NAME, "GDI"
ScreenRes iW, iH, 24

Dim As String sTitle = "GDI Mandelbrot"
WindowTitle sTitle

Dim as HWND hHWND
ScreenControl(FB.GET_WINDOW_HANDLE, Cast(Integer, hHWND))

Dim Shared As HDC hDC
hDC = GetDC(hHWND)

Dim As BITMAPV5HEADER tBIV5HDR
tBIV5HDR.bV5Size = SizeOf(BITMAPV5HEADER)
tBIV5HDR.bV5Width = iW
tBIV5HDR.bV5Height = -iH
tBIV5HDR.bV5Planes = 1
tBIV5HDR.bV5BitCount = 32
tBIV5HDR.bV5Compression = BI_BITFIELDS
tBIV5HDR.bV5AlphaMask = &hFF000000
tBIV5HDR.bV5RedMask = 	&h00FF0000
tBIV5HDR.bV5GreenMask = &h0000FF00
tBIV5HDR.bV5BlueMask = 	&h000000FF

Dim Shared As ULong Ptr aBits
Dim As HBitmap hBitmapGDI
Dim Shared As HDC hGfxDC

hBitmapGDI = CreateDIBSection(hDC, @tBIV5HDR, DIB_RGB_COLORS, @aBits, NULL, NULL)
hGfxDC = CreateCompatibleDC(hDC)
Var hObjOld = SelectObject(hGfxDC, hBitmapGDI)

Dim evt As EVENT
Dim As ULong iFPS = 0, i, iMax = iW * iH - 1 , iX, iY, iARGB


Dim As Double fTimer, fEnd

fTimer = Timer
Mandelbrot(-2, 1, -1, 1, 1000, 100)
fEnd = Timer - fTimer
WindowTitle sTitle  & " / Took " & fEnd & " seconds to render."

Do
	If (ScreenEvent(@evt)) Then
		Select Case evt.Type 
			Case SC_ESCAPE, EVENT_WINDOW_CLOSE
				SelectObject(hGfxDC, hObjOld)
				ReleaseDC(hHWND, hDC)
				DeleteDC(hGfxDC)
				DeleteObject(hBitmapGDI)
				Exit Do
		End Select
	EndIf
	Sleep(60)
Loop

Sub Mandelbrot(xmin As Single, xmax As Single, ymin As Single, ymax As Single, iterations As ULong, iDisplayFrequency As ULong = 100)
	Dim As ULong i, ppos
	Dim As Double c, x, y
	Dim As ULong iX, iY
	For iY = 0 To iH - 1
		For iX = 0 To iW - 1

			x = xmin + (xmax - xmin) * iX / (iW - 1)
			y = ymin + (ymax - ymin) * iY / (iH - 1)
			i = mandelIter(x, y, iterations)
			ppos = (iW * iY + iX)
			
			If (i > iterations) Then
				aBits[ppos] = &hFF000000
			Else
				c = 3 * Log(i) / Log(iterations - 1.0)
				If (c < 1) Then
					aBits[ppos] = &hFF000000 + (Int(255 * c) Shl 16)
				ElseIf ( c < 2 ) Then
					aBits[ppos] = &hFF000000 + (255 Shl 16) + (Int(255 * (c - 1)) Shl 8) 
				Else
					aBits[ppos] = &hFFFFFF00 + (Int(255 * (c - 2)))
				EndIf
			EndIf
			
		Next
		If iY Mod iDisplayFrequency = 0 Then BitBlt(hDC, 0, 0, iW, iH, hGfxDC, 0, 0, SRCCOPY)
	Next
	BitBlt(hDC, 0, 0, iW, iH, hGfxDC, 0, 0, SRCCOPY)
End Sub

Function mandelIter(cx As Single, cy As Single, maxIter As ULong) As ULong
	Dim As Single x = 0.0, y, xx, yy, xy 
	Dim As ULong i = maxIter
	While (xx + yy <= 4 And i > 0)
		xy = x * y
		xx = x * x
		yy = y * y
		x = xx - yy + cx
		y = xy + xy + cy
		i -= 1
	Wend
	Return maxIter - i
End Function